@@ -1,5 +1,5 @@
 /* Fault handler information.  Unix version.
-   Copyright (C) 1993-1999, 2002-2003, 2006, 2008  Bruno Haible <bruno@clisp.org>
+   Copyright (C) 1993-1999, 2002-2003, 2006, 2008-2009  Bruno Haible <bruno@clisp.org>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,6 +15,10 @@
    along with this program; if not, write to the Free Software Foundation,
    Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
+/* Persuade glibc <sys/ucontext.h> to declare macros designating register
+   indices: REG_RSP on x86_64, REG_ESP on i386.  */
+#define _GNU_SOURCE 1
+
 #include "sigsegv.h"
 
 /* On the average Unix platform, we define
@@ -486,8 +490,13 @@ stackoverflow_install_handler (stackoverflow_handler_t handler,
 #else /* HAVE_SIGALTSTACK */
   {
     stack_t ss;
+#if SIGALTSTACK_SS_REVERSED
+    ss.ss_sp = (char *) extra_stack + extra_stack_size - sizeof (void *);
+    ss.ss_size = extra_stack_size - sizeof (void *);
+#else
     ss.ss_sp = extra_stack;
     ss.ss_size = extra_stack_size;
+#endif
     ss.ss_flags = 0; /* no SS_DISABLE */
     if (sigaltstack (&ss, (stack_t*)0) < 0)
       return -1;
